
//Motorola Micom 2E control head retrofit to MCS2000 by C. Crane v2.0
//display used is Noritake DS2045G with built in driver
//Construction details and bug reports at web page below
//http://www.p25.ca/threads/399-Micom-2E-home-made-control-head-project
//Serial interface uses ODD parity at 9600
// * VFD Data to analog pin 2
// * VFD Clock to analog pin 3
// * VFD Chip select to analog pin 4

#include <SoftPWM.h>
#include <SoftPWM_timer.h>
#include "Wire.h"
#include <SPI_VFD.h>
#include <Keypad.h>
#include <EEPROM.h>
//#include "LiquidCrystal.h"  // initialize the library with the numbers of the interface pins LiquidCrystal, lcd(0) for backpack
char fileVersion = 21;
const int sensorMin = 0;      // volume minimum
const int sensorMax = 1023; // volume maximum
const int pttSense = 12;     //
int pttState = 0;
char holdKey;
unsigned long t_hold;
const byte ROWS = 5; //four rows
const byte COLS = 5; //four columns
// mapped to MCS2000 control head custom PC board layout per V2A design file
char keys[ROWS][COLS] = {
  {
    '1','2','3','MENU','MORE'                                                        }
  ,
  {
    '4','5','6','Pesc','z'                                                        }
  ,   // z in place of F4 due to conflicts in compiler
  {
    '7','8','9','Enter','(f_three)'                                                        }
  , // f_three in place of Fthree due to confilcts in compiler
  {
    '*','0','#','SQL','Ftwo'                                                        }
  ,
  {
    'Fone', 'upperSb', 'LowerSB', 'arrowDown', 'arrowuP'                                             }
};


byte colPins[COLS] = {
  2, 3, 4, 5, 6}; //connect to the column pinouts of the keypad
byte rowPins[ROWS] = {
  7, 8, 9, 10, 11}; //connect to the row pinouts of the keypad
Keypad keypad = Keypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS );
SPI_VFD vfd(A2, A3, A4 );
uint8_t brightness = 0;
//LiquidCrystal lcd(0);  // (0) for serial backpack use, otherwise refer to the basic LCD parallel hookup
// instructions in the LCD example code in the Arduino IDE for wiring diagram
// CLK pin to Analog 5 (i2c SCL pin) and DAT to Analog 4 (i2c SDA pin)
//Custom Icons
byte squelchOnChar[8] = {
  B00011,
  B00111,
  B11111,
  B11111,
  B00111,
  B00011,
  B00000,
  B00000
};

byte muteChar[8] = {
  B00000,
  B01110,
  B11111,
  B11011,
  B11111,
  B01110,
  B00000,
  B00000
};

byte notchFilterChar[8] = {
  B01001,
  B01101,
  B01011,
  B01001,
  B00000,
  B10100,
  B10100,
  B11111
};

byte clarifierChar[8] = {
  B00000,
  B01010,
  B10001,
  B10101,
  B10001,
  B01010,
  B00000,
  B00000
};

byte narFilter[8] = {
  B01110,
  B01010,
  B01010,
  B01010,
  B01010,
  B01010,
  B11011,
  B00000
};

byte more[8] = {
  B00000,
  B01000,
  B01100,
  B01110,
  B01100,
  B01000,
  B00000,
  B00000
};

byte leftNotch[8] = {
  B10111,
  B10100,
  B10100,
  B10100,
  B10100,
  B10100,
  B11100,
  B00000
};

byte rightNotch[8] = {
  B11101,
  B00101,
  B00101,
  B00101,
  B00101,
  B00101,
  B00111,
  B00000
};

byte topLine[8] = {
  B11111,
  B00000,
  B00000,
  B00000,
  B00000,
  B00000,
  B00000,
  B00000
};

char freqLine[10];
char first;
char sum=0;
char length;
char brightnessKeypad;
boolean ackfilter = false;
boolean checkSum = false;
byte opCode;
byte subopCode;
byte endMarker;
byte volumeVar = 0x00;
byte previousvolume;
int a;
int b;
int c = 0;
int i;
int j;
int m;
int n;
int led = 13;
int index;
long previousMillis = 0;
long interval = 3000; //volume indication fade out time
unsigned long time;
unsigned long prev_time;
boolean blink_state;

//ICONS state
    int squelch;
    int i_usb;
    int noiseBlanker;
    int notchFilter;
    int fastAGC;
    int i_lsb;
    int mute;
    int clarifier;
    int i_narFilter;
    int stack;
    int i_more;

boolean tuning = false;
boolean pttLogic = false;
boolean packetRead = false;
boolean keypadBrightness1 = false;
//Control data strings
byte volumestr[8] = {
  0x24,0x03,0x18,0x2b,0x00,0x00,0x6A,0x03};
byte squelchOff[8] = {
  0x24,0x03,0x18,0x09,0x00,0x00,0x48,0x03};
byte squelchOn[8] = {
  0x24,0x03,0x18,0x09,0x00,0x01,0x49,0x03};
byte volume[8] =  {
  0x24,0x03,0x18,0x2B,0x00,0x00,0x6A,0x03}; // volume level 0 in hex
byte reportVol[7] = {
  0x24,0x01,0x18,0x2C,0x69,0x03};
byte button1[10] = {
  0x24,0x05,0x18,0x36,0xFF,0x79,0xEF,0x01,0xDF,0x03}; // keypad button 1
byte button2[10] = {
  0x24,0x05,0x18,0x36,0xFF,0x3B,0xEF,0x01,0xA1,0x03}; // keypad button 2
byte button3[10] = {
  0x24,0x05,0x18,0x36,0xFF,0x7B,0xE7,0x01,0xD9,0x03}; // keypad button 3
byte button4[10] = {
  0x24,0x05,0x18,0x36,0xFF,0x5B,0xEF,0x01,0xC1,0x03}; // keypad button 4
byte button5[10] = {
  0x24,0x05,0x18,0x36,0xFF,0x7B,0xEB,0x01,0xDD,0x03}; // keypad button 5
byte button6[10] = {
  0x24,0x05,0x18,0x36,0xFF,0x7B,0x6F,0x01,0x61,0x03}; // keypad button 6
byte button7[10] = {
  0x24,0x05,0x18,0x36,0xFF,0x7B,0xAF,0x01,0xA1,0x03}; // keypad button 7
byte button8[10] = {
  0x24,0x05,0x18,0x36,0xF7,0x7B,0xEF,0x01,0xD9,0x03}; // keypad button 8
byte button9[10] = {
  0x24,0x05,0x18,0x36,0xFF,0x7A,0xEF,0x01,0xE0,0x03}; // keypad button 9
byte button0[10] = {
  0x24,0x05,0x18,0x36,0x7F,0x7B,0xEF,0x01,0x61,0x03}; // keypad button 0
byte buttonAsterisk[10] = {
  0x24,0x05,0x18,0x36,0xFB,0x7B,0xEF,0x01,0xDD,0x03}; // keypad button *
byte buttonPound[10] = {
  0x24,0x05,0x18,0x36,0xFF,0x6B,0xEF,0x01,0xD1,0x03}; // keypad button #
byte buttonMENU[10] = {
  0x24,0x05,0x18,0x36,0xFF,0x7B,0xEF,0x00,0xE0,0x03}; // keypad button MENU
byte buttonPesc[10] = {
  0x24,0x05,0x18,0x36,0xEF,0x7B,0xEF,0x01,0xD1,0x03}; // keypad button Pesc
byte buttonEnter[10] = {
  0x24,0x05,0x18,0x36,0xFF,0x7B,0xED,0x01,0xDF,0x03}; // keypad button Enter
byte colorKey[10] = {
}; // keypad button for color or other use
byte moreKey[10] = {
  0x24,0x05,0x18,0x36,0xFD,0x7B,0xEF,0x01,0xDF,0x03};  // more key
byte fOneKey[10] = {
  0x24,0x05,0x18,0x36,0xBF,0x7B,0xEF,0x01,0xA1,0x03};  // soft key F1
byte fTwoKey[10] = {
  0x24,0x05,0x18,0x36,0xFF,0x73,0xEF,0x01,0xD9,0x03};  // soft key F2
byte fThreeKey[10] = {
  0x24,0x05,0x18,0x36,0xFF,0x7B,0xEE,0x01,0xE0,0x03};  //soft key F3
byte fFourKey[10] = {
  0x24,0x05,0x18,0x36,0xFF,0x7B,0xCF,0x01,0xc1,0x03};  // soft key F4
byte buttonRelease[10] = {
  0x24,0x05,0x18,0x36,0xFF,0x7B,0xEF,0x01,0xE1,0x03}; // button release command
byte arrowUp[10] = {
  0x24,0x05,0x18,0x36,0xFE,0x7B,0xEF,0x01,0xE0,0x03}; // up button command
byte arrowDown[10] = {
  0x24,0x05,0x18,0x36,0xDF,0x7B,0xEF,0x01,0xC1,0x03}; // down button command
byte lsb[8] = {
  0x24,0x03,0x18,0x03,0x00,0x00,0x42,0x03};
byte usb[8] = {
  0x24,0x03,0x18,0x03,0x00,0x01,0x43,0x03};
byte specialReports[9] = {
  0x24,0x04,0x18,0x29,0x00,0x01,0x2E,0x98,0x03};  // enable serial data reporting
byte pttEngage[7] = {
  0x24, 0x02, 0x81, 0x13, 0x01, 0xBB, 0x03};
byte pttRelease[7] = {
  0x24, 0x02, 0x81, 0x14, 0x01, 0xBC, 0x03};
byte acknowledge[6] = {
  0x24,0x01,0x10,0xF3,0x28,0x03}; // ack to radio from head
byte radioack[6] = {
  0x24,0x01,0x80,0xF3,0x98,0x03};  // ack from radio to head
//byte rxACK[6] = {
//  0x24,0x01,0x80,0xF3,0x98,0x03}; // ack from radio
//byte txACK[6] = {
//  0x24,0x01,0x10,0xF3,0x28,0x03}; // ack to radio

//    char rxdata[50];


void setup() {  //-------------------------------SETUP-----------------------------------------------

  SoftPWMBegin();
  //SoftPWMSet(A5, 1000);
  brightnessKeypad = EEPROM.read(c);
  SoftPWMSetPercent(A5, brightnessKeypad); // keypad brightness control
  pinMode(12, INPUT_PULLUP);
  pinMode(pttSense, INPUT);
  vfd.begin(20, 4);  //Using Noratake DS2045G all in one VFD display
  //lcd.begin(20, 4);
  //lcd.setBacklight(HIGH);
//  Serial.begin(9600); //changed to 9600 (I swear this worked at 19,200 at one time)
  Serial.begin(9600, SERIAL_8O1);
  keypad.setHoldTime(500);
  UCSR0C = UCSR0C | B00111000; // line added to change to odd parity so it matches the Micom standard of 8-odd-
  vfd.setCursor(6,1);  // for SELF TEST splash screen on powerup
  vfd.createChar(0, squelchOnChar);
  vfd.createChar(1, narFilter);
  vfd.createChar(2, notchFilterChar);
  vfd.createChar(3, clarifierChar);
  vfd.createChar(4, topLine);
  vfd.createChar(5, leftNotch);
  vfd.createChar(6, rightNotch);
  vfd.createChar(7, more);
  vfd.setCursor(1,0);
  vfd.print("FW Ver. V2A(VAS.1)");
  vfd.setCursor(10,1);
  vfd.print(fileVersion, DEC);
  vfd.setCursor(5,3);
  vfd.print("MICOM-MCS2000");  
  delay(1000);
  vfd.clear();
  Serial.write(specialReports, 9); // causes radio to report information and talk over serial
}

void loop() {//------------------------LOOP-------------------------------------------------------------
  SoftPWMSetPercent(A5, brightnessKeypad); // keypad brightness control

  brightness = c;
  vfd.setBrightness(brightness++%4);
  //-----------------------------------INITIALIZE-------------------------------------------------------
  time = millis();
//  if (time == 1000){
//    Serial.write(specialReports, 9); // causes radio to report information and talk over serial
//  }
if (prev_time+500<time) {
  if (blink_state) blink_state = false; else blink_state = true;
  prev_time = time;
  if (blink_state) digitalWrite(led, HIGH); else digitalWrite(led, LOW);
}
  //------------------------------------SERIAL READ-----------------------------------------------------

  if (Serial.available() > 0) {

    delay(5);
    first = Serial.read();
    if (first == 0x24) {
      length = Serial.read();
    }
    a = length + 3;    //1
    sum=0;      // reset sum to zero
    char rxdata[50];
    byte count;
    Serial.setTimeout(20);
    count = Serial.readBytes(rxdata, a);

    for (b =0; b <a-2; b ++)  //sum loop
    {
      sum += rxdata[b];
    }
    char checkSum = sum + length + first;
    char checkSumByte = rxdata[length+1];
    endMarker = rxdata[a-1];
    opCode = rxdata[1];
    subopCode = rxdata[2];
    if (checkSum == checkSumByte) {
      checkSum = true;
//      digitalWrite(led, HIGH);
    }
    //if (checkSum /= checkSumByte) {
    //   digitalWrite(led, LOW);
    // }
    //--------------------serial data checksum filters and discrimination----------------------------------------


    if (first == 0x24 && endMarker == 0x03 && checkSum == true) {
      packetRead = true;
      Serial.write(acknowledge, 6);
    }

    if (opCode == 0xF3 && checkSum == true)
    {
      ackfilter = true;
    }
    else
    {
      ackfilter =false;
    }
    if (packetRead == true && ackfilter == false) {
    }


//  vfd.clear();
//  for (i=0; i<40; i++) {
//    if (i < 10) vfd.setCursor(i*2,0);
//    if (i >= 10 && i < 20) vfd.setCursor((i-10)*2,1);
//    if (i >= 20 && i < 30) vfd.setCursor((i-20)*2,2);
//    if (i >= 30 && i < 40) vfd.setCursor((i-30)*2,3);
//    vfd.print(rxdata[i], HEX);
//  }
//
//delay(300);
//
    //---------FREQUENCY(LINE1)-----------------------------
    if (opCode == 0x2E && subopCode == 0x03 && checkSum == true)
    {
      //EXAMPLE FREQUENCY PACKET 24 0F 81 2E 03 02 6E 46 20 20 38 2C 39 39 32 2E 39 38 82 03 (display 0, 1)
      char freqLine[11];
      for(int i=5, j=0; i<=15; i++, j++) {
        freqLine[j] = rxdata[i];
        freqLine[j+1] = '\0';
      }
      //Serial.print(freqLine);
      vfd.setCursor(0,1);
      vfd.print(freqLine);
      Serial.flush();
    }
    //----------------ALE scan(LINE 1)--------------------
    if (opCode == 0x2E && subopCode == 0x0D && checkSum == true)
    {
      vfd.setCursor(7,1);
      vfd.print("             ");
      // byte ale;

      //byte disp;
      //ale = rxdata[4];
      byte scan = byte(rxdata[4]);
      //disp = ((ale/100) + ((ale/10)%10) + (ale%10));   // hex to decimal conversion for ALE "fast scan"
      // bug in conversion, ale net 2 11-16 was skewed one high and indicated 12-17, single digits appear to work
      
      vfd.setCursor(7,1);
//      if (scan>=100) vfd.setCursor(7,1);
//      if (scan>10 && scan<100) vfd.setCursor(6,1);
//      if (scan<10) vfd.setCursor(5,1);
      //vfd.print(disp);
      //vfd.print(ale, DEC);
      vfd.print(scan, DEC);
      //vfd.print(" ");
    }
    //---------------------MENU(LINE0)------------------------------
    if (opCode == 0x2E && subopCode == 0x02 && checkSum == true)
    {
      //EXAMPLE MENU PACKET 24 09 81 2E 02 02 55 4D 45 4E 55 20 8A 03 (display 0, 0)
      char freqLine[11];
      for(int i=5, j=0; i<=9; i++, j++) {
        freqLine[j] = rxdata[i];
        freqLine[j+1] = '\0';
      }
      vfd.setCursor(0,0);
      vfd.print(freqLine);
      Serial.flush();
    }
    // ---------------------SOFT BUTTONS(LINE3)-------------------------
    if (opCode == 0x2E && subopCode == 0x04 && checkSum == true)
    {
      char freqLine[20];
      for(int i=5, j=0; i<=19; i++, j++) {
        freqLine[j] = rxdata[i];
        freqLine[j+1] = '\0';
      }
      freqLine[18] = freqLine[15];
      freqLine[17] = freqLine[14];
      freqLine[16] = freqLine[13];
      freqLine[15] = freqLine[12];
      freqLine[14] = ' ';
      freqLine[13] = freqLine[11];
      freqLine[12] = freqLine[10];
      freqLine[11] = freqLine[9];
      freqLine[10] = freqLine[8];
      freqLine[9] = ' ';
      freqLine[8] = freqLine[7];
      freqLine[7] = freqLine[6];
      freqLine[6] = freqLine[5];
      freqLine[5] = freqLine[4];
      freqLine[4] = ' ';
      vfd.setCursor(1,3);
      vfd.print(freqLine);
      Serial.flush();
    }
    //----------------------------ICONS-------------------------------
    int txLed;
    int remoteLed;
    int icons;
//    int usb;
//    int noiseBlanker;
//    int notchFilter;
//    int fastAGC;
//    int lsb;
//    int mute;
//    int clarifier;
//    int narFilter;
//    int stack;
//    int more;
    int leftNotch;
    int rightNotch;
    int topLine;
    int lowestBar;
    int secondBar;
    int thirdBar;
    int fourthBar;
    int fifthBar;
    int sixthBar;
    int seventhBar;
    int eighthBar;
    int ninthBar;
    int tenthBar;
    if (opCode == 0x2E && subopCode == 0x05)
    {
      i_usb = bitRead(rxdata[4], 7);
//      i_usb = bitRead(rxdata[4], 7)*2+bitRead(rxdata[4], 6);
      squelch = bitRead(rxdata[4], 5);
//      squelch = bitRead(rxdata[4], 5)*2+bitRead(rxdata[4], 4);
      noiseBlanker = bitRead(rxdata[4], 3)*2+bitRead(rxdata[4], 2);
      notchFilter = bitRead(rxdata[4], 1)*2+bitRead(rxdata[4], 0);
      fastAGC = bitRead(rxdata[5], 7)*2+bitRead(rxdata[5], 6);
      i_lsb = bitRead(rxdata[5], 5);
//      i_lsb = bitRead(rxdata[5], 5)*2+bitRead(rxdata[5], 4);
      mute = bitRead(rxdata[5], 3)*2+bitRead(rxdata[5], 2);
      clarifier = bitRead(rxdata[5], 1)*2+bitRead(rxdata[5], 0);
      i_narFilter = bitRead(rxdata[6], 7)*2+bitRead(rxdata[6], 6);
      stack = bitRead(rxdata[6], 5)*2+bitRead(rxdata[6], 4);
      i_more = bitRead(rxdata[6], 3)*2+bitRead(rxdata[6], 2);
  }
      // upper side band icon  -----------
      if (i_usb == 1) {
        vfd.setCursor(17,0);
        vfd.print("USB");
      }
      //Squelch icons--------------------
      if (squelch == 0) {
        vfd.setCursor(14,0);
        vfd.print(" ");
      }
      if (squelch == 1) {
        vfd.setCursor(14,0);
        vfd.write(0);
      }
      //Noise Blanker icon---------------
      if (noiseBlanker == 3) {
        vfd.setCursor(11,0);
        if (blink_state) vfd.print("NB"); else vfd.print("  ");
      }
      if (noiseBlanker == 2) {
        vfd.setCursor(11,0);
        vfd.print("NB");
      }
      if (noiseBlanker == 1) {
        vfd.setCursor(11,0);
        vfd.print("  ");
      }
      //Notch Filter icon----------------
      if (notchFilter == 3) {
        vfd.setCursor(10,0);
        if (blink_state) vfd.write(2); else vfd.print(" ");
      }
      if (notchFilter == 2) {
        vfd.setCursor(10,0);
        vfd.write(2);
      }
      if (notchFilter == 1) {
        vfd.setCursor(10,0);
        vfd.print(" ");
      }
      // Fast AGC icon-------------------
      if (fastAGC == 3) {
        vfd.setCursor(9,0);
        if (blink_state) vfd.print("F"); else vfd.print(" ");
      }
      if (fastAGC == 2) {
        vfd.setCursor(9,0);
        vfd.print("F");
      }
      if (fastAGC == 1) {
        vfd.setCursor(9,0);
        vfd.print(" ");
      }
      // lower side band icon------------
      if (i_lsb == 1) {
        vfd.setCursor(17,0);
        vfd.print("LSB");
        // mute icon
      }
      // mute icon            // not yet used due to custom char space and lack of need
//      if (mute == 1) {
//        vfd.setCursor(10,0);
//        vfd.write(1);
//      }
//      if (mute == 0) {
//        vfd.setCursor(10,0);
//        vfd.print(" ");
//      }
      // clarifier icon------------------
      if (clarifier == 3) {
        vfd.setCursor(13,0);
        if (blink_state) vfd.write(3); else vfd.print(" ");
      }
      if (clarifier == 2) {
        vfd.setCursor(13,0);
        vfd.write(3);
      }
      if (clarifier == 1) {
        vfd.setCursor(13,0);
        vfd.print(" ");
      }
      // narrow filter icon--------------
      if (i_narFilter == 3) {
        vfd.setCursor(8,0);
        if (blink_state) vfd.write(1); else vfd.print(" ");
      }
      if (i_narFilter == 2) {
        vfd.setCursor(8,0);
        vfd.write(1);
      }
      if (i_narFilter == 1) {
        vfd.setCursor(8,0);
        vfd.print(" ");
      }
      // More icon >---------------------
      if (i_more == 3) {
        vfd.setCursor(19,3);
        if (blink_state) vfd.write(7); else vfd.print(" ");
      }
      if (i_more == 2) {
        vfd.setCursor(19,3);
        vfd.write(7);
      }
      if (i_more == 1) {
        vfd.setCursor(19,3);
        vfd.print(" ");
      }
//    }
    //----------------------------------RSSI(line2)---------------------------------------
    if (opCode == 0x2E && subopCode == 0x06)
    {
      if (rxdata[4] == 0x00 && rxdata[5] == 0x00){
        vfd.setCursor(0,2);
        vfd.print("     ");
        vfd.setCursor(5,2);
        vfd.print("         ");
      }

      if (rxdata[5] == 0x01){

        vfd.setCursor(0,2);
        vfd.print("S>   ");
        vfd.setCursor(3,2);
        vfd.print("1");
        vfd.setCursor(5,2);
        vfd.print("-        ");
      }

      if (rxdata[5] == 0x03){
        vfd.setCursor(0,2);
        vfd.print("S>   ");
        vfd.setCursor(3,2);
        vfd.print("2");
        vfd.setCursor(5,2);
        vfd.print("--       ");
      }

      if (rxdata[5] == 0x07){
        vfd.setCursor(0,2);
        vfd.print("S>   ");
        vfd.setCursor(3,2);
        vfd.print("3");
        vfd.setCursor(5,2);
        vfd.print("---      ");
      }
      if (rxdata[5] == 0x0F){
        vfd.setCursor(0,2);
        vfd.print("S>   ");
        vfd.setCursor(3,2);
        vfd.print("4");
        vfd.setCursor(5,2);
        vfd.print("----     ");
      }
      if (rxdata[5] == 0x1F){
        vfd.setCursor(0,2);
        vfd.print("S>   ");
        vfd.setCursor(3,2);
        vfd.print("5");
        vfd.setCursor(5,2);
        vfd.print("-----    ");
      }
      if (rxdata[5] == 0x3F){
        vfd.setCursor(0,2);
        vfd.print("S>   ");
        vfd.setCursor(3,2);
        vfd.print("6");
        vfd.setCursor(5,2);
        vfd.print("------   ");
      }
      if (rxdata[5] == 0x7F){
        vfd.setCursor(0,2);
        vfd.print("S>   ");
        vfd.setCursor(3,2);
        vfd.print("7");
        vfd.setCursor(5,2);
        vfd.print("-------  ");
      }
      if (rxdata[5] == 0xFF){
        vfd.setCursor(0,2);
        vfd.print("S>   ");
        vfd.setCursor(3,2);
        vfd.print("8");
        vfd.setCursor(5,2);
        vfd.print("-------- ");
      }
      if (rxdata[4] == 0x01 && rxdata[5] == 0xFF){
        vfd.setCursor(0,2);
        vfd.print("S>  ");
        vfd.setCursor(3,2);
        vfd.print("9");
        vfd.setCursor(5,2);
        vfd.print("--------+");
      }
    }
  } //end of (if serial available)
  else
  {
//    digitalWrite(led, LOW);
  }
  //-----------------------------------digital PTT option and sense------------------------------------------------------------
  int pttInput = digitalRead(12);
  if (pttInput == HIGH && pttLogic == true) {  //HIGH
    //Serial.write(pttEngage, 7);
    pttLogic = false;
    vfd.setCursor(18,2);
    vfd.print("Tx");
  }
  if (pttInput == LOW && pttLogic == false) {  //LOW
    //Serial.write(pttRelease, 7);
    pttLogic = true;
    vfd.setCursor(18,2);
    vfd.print("  ");
  }
  //pttState = digitalRead(pttSense);

  //  if (pttState == HIGH) {
  // turn LED on:
  //   digitalWrite(txLed, HIGH);
  // }
  //else {
  // turn LED off:
  //  digitalWrite(txLed, LOW);
  //}
  //-----------------------------------ANALOG VOLUME---------------------------
  // analog volume comment out here on down if keypad volume is used
  unsigned long currentMillis = millis();
  int sensorReading = analogRead(A0);   // map the sensor range, Micom goes from 0x00 to 0xFF
  int range = map(sensorReading, sensorMin, sensorMax, 0, 255);
  volumeVar = range;
  volume[4] = volumeVar;
  volume[6] = volumeVar + 0x6A;
  if (previousvolume < range-1 || previousvolume > range+1) {
    //while (previousvolume != volumeVar){ // not used due to erratic, continuous sampling
    Serial.write(volume, 8);
    //Serial.write(reportVol,7);   // uncomment if radio volume report is to be used later on
    vfd.setCursor(15,1);
    vfd.print("V");
    vfd.setCursor(17,1);
    vfd.print(volumeVar);
    if (volumeVar < 100){
      vfd.setCursor(19,1);
      vfd.print(" ");
    }
    if (volumeVar < 10){
      vfd.setCursor(18,1);
      vfd.print("  ");
    }
    previousvolume = volumeVar;
  }
  else
  {
    if(currentMillis - previousMillis > interval) {  //volume fade out
      previousMillis = currentMillis;  // save the last time you acted
      vfd.setCursor(15,1);
      vfd.print("      ");
    }
  }
  // comment out from here on up if keypad volume is used

  //-------------------------------------KEYPAD AND BUTTONS-------------------------------

  char key = keypad.getKey();
  if(key) {
    holdKey = key;
  }
  switch (key) {
  case '1':
    Serial.write(button1, 10);
    Serial.write(buttonRelease, 10);

    break;
  case '2':
    Serial.write(button2, 10);
    Serial.write(buttonRelease, 10);

    break;
  case '3':
    Serial.write(button3, 10);
    Serial.write(buttonRelease, 10);

    break;
  case '4':
    Serial.write(button4, 10);
    Serial.write(buttonRelease, 10);

    break;
  case '5':
    Serial.write(button5, 10);
    Serial.write(buttonRelease, 10);

    break;
  case '6':
    Serial.write(button6, 10);
    Serial.write(buttonRelease, 10);

    break;
  case '7':
    Serial.write(button7, 10);
    Serial.write(buttonRelease, 10);

    break;
  case '8':
    Serial.write(button8, 10);
    Serial.write(buttonRelease, 10);

    break;
  case '9':
    Serial.write(button9, 10);
    Serial.write(buttonRelease, 10);

    break;
  case '0':
    Serial.write(button0, 10);
    Serial.write(buttonRelease, 10);

    break;
  case '*':
    Serial.write(buttonAsterisk, 10);
    Serial.write(buttonRelease, 10);

    break;
  case '#':
    Serial.write(buttonPound, 10);
    Serial.write(buttonRelease, 10);

    break;
  case 'MENU':
    Serial.write(buttonMENU, 10);
    delay(44);
    //Serial.write(txACK,sizeof(txACK));
    Serial.write(buttonRelease, 10);

    break;
  case 'Pesc':
    Serial.write(buttonPesc, 10);
    Serial.write(buttonRelease, 10);

    break;
  case 'SQL':

    // Serial.write(GetData,sizeof(GetData));


    if (squelch == 1){
      Serial.write(squelchOff, 8);
    }
    else
      if (squelch == 0) {
        Serial.write(squelchOn, 8);
      }
    break;

  case 'Enter':
    Serial.write(buttonEnter, 10);
    Serial.write(buttonRelease, 10);
    //Serial.write(txACK,sizeof(txACK));
    //Serial.write(GetData,sizeof(GetData));
    break;

  case 'MORE':
    Serial.write(moreKey, 10);
    //delay(20);
    Serial.write(buttonRelease, 10);
    Serial.write(specialReports,9);

    break;

  case 'Fone':
    Serial.write(fOneKey, 10);
    Serial.write(buttonRelease, 10);
    break;

  case 'Ftwo':
    Serial.write(fTwoKey, 10);
    Serial.write(buttonRelease, 10);
    break;

  case '(f_three)':
    Serial.write(fThreeKey, 10);
    Serial.write(buttonRelease, 10);
    break;

  case 'z':
    Serial.write(fFourKey, 10);
    Serial.write(buttonRelease, 10);
    break;

  case 'arrowUP':
    tuning = true;
    Serial.write(arrowUp, 10);
    //delay(44);
    //Serial.write(txACK,sizeof(txACK));
    Serial.write(buttonRelease, 10);
    break;

  case 'arrowDown':
    tuning = true;
    Serial.write(arrowDown, 10);
    // delay(44);
    //Serial.write(txACK,sizeof(txACK));
    Serial.write(buttonRelease, 10);
    break;

  case 'lowerSB':
    tuning = true;
    Serial.write(lsb, 8);
    Serial.write(buttonRelease, 10);
    break;

  case 'upperSb':
    Serial.write(usb, 8);
    Serial.write(buttonRelease, 10);
    break;
  }

  if (keypad.getState() == HOLD) {  //---------------FAST SCROLL---------------------
    if ((millis() - t_hold) > 200 ) {
      switch (holdKey) {

      case 'arrowUP':
        tuning = true;

        Serial.write(arrowUp, 10);
        // if (packetRead == true && ackfilter == false){
        Serial.write(buttonRelease, 10);
        //}
        break;

      case 'arrowDown':
        tuning = true;

        Serial.write(arrowDown, 10);
        // if (packetRead == true && ackfilter == false){
        Serial.write(buttonRelease, 10);
        //}
        break;
      case '0':
        keypadBrightness1 = true;
        break;

        // ---------------------------------TO BE USED WITH ROCKER TYPE VOLUME--------------------
      case 'volUp':
        volume[4] = volumeVar;
        volume[6] = volumeVar + 0x6A;
        volumeVar = volumeVar + 0x02;
        Serial.write(volume, 8);
        Serial.write(reportVol,7);
        // vfd.setCursor(13,1);
        // vfd.print("Vol");
        // vfd.setCursor(17,1);
        // vfd.print(volumeVar, HEX);
        break;

      case 'volD':
        volume[4] = volumeVar;
        volume[6] = volumeVar + 0x6A;
        volumeVar = volumeVar - 0x02;
        Serial.write(volume, 8);
        Serial.write(reportVol,7);
        //vfd.setCursor(13,1);
        // vfd.print("Vol");
        // vfd.setCursor(17,1);
        // vfd.print(volumeVar, HEX);
        break;
      }
      t_hold = millis();
    }
    else
      keypadBrightness1 = false;
  }
  if (keypadBrightness1 == true) {
    brightnessKeypad = volumeVar;
    EEPROM.write(c,volumeVar);
  }
} //loop end


























